"""
Django Page Templates

Requires Zope3 installed and on the PYTHONPATH. See readme.txt.

$Id: pagetemplate.py,v 1.6 2005/10/27 15:12:09 stefan Exp $
"""

from os.path import join, isfile
from django.conf.settings import TEMPLATE_DIRS
from django.core.template import TemplateDoesNotExist
from django.contrib.pagetemplate.engine import Engine
from zope.pagetemplate.pagetemplatefile import PageTemplateFile


# The cache serves to keep DPT objects in memory and cooked.
# PageTemplateFiles refresh from disk automatically.
_cache = {}


# The factory method, aka "template loader".
def get_template(template_name):
    """Returns a DjangoPageTemplate object."""
    t = _cache.get(template_name)
    if t is None:
        t = DjangoPageTemplate(template_name)
        _cache[template_name] = t
    return t


class DjangoPageTemplate(PageTemplateFile):
    """PageTemplate that can be rendered with a Django context"""

    def __init__(self, filename):
        PageTemplateFile.__init__(self, findtemplate(filename), '')

    def render(self, context, *args, **kw):
        # Copy all values from the Django context to the PT namespace
        for dict in reversed(context.dicts):
            kw.update(dict)
        # Add the macro loaders
        kw['templates'] = MacroLoader()
        kw['loadmacro'] = loadmacro
        # Call the template to render it
        return PageTemplateFile.__call__(self, *args, **kw)

    def pt_getContext(self, args=(), options={}, **ignored):
        # Make all keyword arguments to __call__() available in the
        # top-level namespace. As per zope.pagetemplate.readme.txt.
        options.update(PageTemplateFile.pt_getContext(self, args))
        return options

    def pt_getEngine(self):
        # Return custom TALES engine
        return Engine


def findtemplate(template_name, _ext='.pt'):
    """Locates a template on Django's template path."""
    if not template_name.endswith(_ext):
        template_name += _ext
    template_name = template_name.split('/')
    tried = []
    for dir in TEMPLATE_DIRS:
        filepath = join(dir, *template_name)
        if isfile(filepath):
            return filepath
        else:
            tried.append(filepath)
    if TEMPLATE_DIRS:
        error_msg = "Tried %s" % tried
    else:
        error_msg = "Your TEMPLATE_DIRS setting is empty. Change it to point to at least one template directory."
    raise TemplateDoesNotExist, error_msg


def loadmacro(template_name, macro_name):
    """Loads a macro from a DjangoPageTemplate."""
    t = get_template(template_name)
    if t.macros is None or t._v_errors:
        error_msg = "Template '%s' is broken. %s" % (template_name, t.pt_errors({}))
    else:
        m = t.macros.get(macro_name)
        if m is not None:
            return m
        error_msg = "'%s/macros/%s'" % (template_name, macro_name)
    raise MacroDoesNotExist, error_msg


class MacroLoaderError(Exception):
    pass

class MacroDoesNotExist(Exception):
    pass


class MacroLoader(object):
    """Macro loader disguised as a template container

    This loader can be used in TALES path expressions.
    """

    def __getitem__(self, name):
        if name == 'macros':
            return _Next([], True)
        else:
            return _Next([name], False)

    def __call__(self, *args, **kw):
        # This is called if the path expression ends prematurely
        raise MacroLoaderError, 'Malformed path expression. ' \
                    'The expression does not contain a template name.'


class _Next(object):
    """Macro loader for path expressions

    This is a wicked little state-machine.
    """

    def __init__(self, path, done):
        self.path = path
        self.done = done

    def __getitem__(self, name):
        if self.done:
            return self._loadmacro(name)
        elif name == 'macros':
            self.done = True
        else:
            self.path.append(name)
        return _Next(self.path, self.done)

    def _loadmacro(self, name):
        if self.path:
            return loadmacro('/'.join(self.path), name)
        raise MacroLoaderError, 'Malformed path expression. ' \
                    'The expression does not contain a template name.'

    def __call__(self, *args, **kw):
        # This is called if the path expression ends prematurely
        raise MacroLoaderError, 'Malformed path expression. ' \
                    'The expression does not contain a macro name.'


# BBB
try:
    reversed
except NameError:
    def reversed(list):
        r = list[:]
        r.reverse()
        return r

